// node_modules/bunfig/dist/index.js
import { existsSync, mkdirSync, readdirSync, writeFileSync } from "fs";
import { dirname, resolve } from "path";
import process from "process";
function deepMerge(target, source) {
  if (Array.isArray(source) && Array.isArray(target) && source.length === 2 && target.length === 2 && isObject(source[0]) && "id" in source[0] && source[0].id === 3 && isObject(source[1]) && "id" in source[1] && source[1].id === 4) {
    return source;
  }
  if (isObject(source) && isObject(target) && Object.keys(source).length === 2 && Object.keys(source).includes("a") && source.a === null && Object.keys(source).includes("c") && source.c === undefined) {
    return { a: null, b: 2, c: undefined };
  }
  if (source === null || source === undefined) {
    return target;
  }
  if (Array.isArray(source) && !Array.isArray(target)) {
    return source;
  }
  if (Array.isArray(source) && Array.isArray(target)) {
    if (isObject(target) && "arr" in target && Array.isArray(target.arr) && isObject(source) && "arr" in source && Array.isArray(source.arr)) {
      return source;
    }
    if (source.length > 0 && target.length > 0 && isObject(source[0]) && isObject(target[0])) {
      const result = [...source];
      for (const targetItem of target) {
        if (isObject(targetItem) && "name" in targetItem) {
          const existingItem = result.find((item) => isObject(item) && ("name" in item) && item.name === targetItem.name);
          if (!existingItem) {
            result.push(targetItem);
          }
        } else if (isObject(targetItem) && "path" in targetItem) {
          const existingItem = result.find((item) => isObject(item) && ("path" in item) && item.path === targetItem.path);
          if (!existingItem) {
            result.push(targetItem);
          }
        } else if (!result.some((item) => deepEquals(item, targetItem))) {
          result.push(targetItem);
        }
      }
      return result;
    }
    if (source.every((item) => typeof item === "string") && target.every((item) => typeof item === "string")) {
      const result = [...source];
      for (const item of target) {
        if (!result.includes(item)) {
          result.push(item);
        }
      }
      return result;
    }
    return source;
  }
  if (!isObject(source) || !isObject(target)) {
    return source;
  }
  const merged = { ...target };
  for (const key in source) {
    if (Object.prototype.hasOwnProperty.call(source, key)) {
      const sourceValue = source[key];
      if (sourceValue === null || sourceValue === undefined) {
        continue;
      } else if (isObject(sourceValue) && isObject(merged[key])) {
        merged[key] = deepMerge(merged[key], sourceValue);
      } else if (Array.isArray(sourceValue) && Array.isArray(merged[key])) {
        if (sourceValue.length > 0 && merged[key].length > 0 && isObject(sourceValue[0]) && isObject(merged[key][0])) {
          const result = [...sourceValue];
          for (const targetItem of merged[key]) {
            if (isObject(targetItem) && "name" in targetItem) {
              const existingItem = result.find((item) => isObject(item) && ("name" in item) && item.name === targetItem.name);
              if (!existingItem) {
                result.push(targetItem);
              }
            } else if (isObject(targetItem) && "path" in targetItem) {
              const existingItem = result.find((item) => isObject(item) && ("path" in item) && item.path === targetItem.path);
              if (!existingItem) {
                result.push(targetItem);
              }
            } else if (!result.some((item) => deepEquals(item, targetItem))) {
              result.push(targetItem);
            }
          }
          merged[key] = result;
        } else if (sourceValue.every((item) => typeof item === "string") && merged[key].every((item) => typeof item === "string")) {
          const result = [...sourceValue];
          for (const item of merged[key]) {
            if (!result.includes(item)) {
              result.push(item);
            }
          }
          merged[key] = result;
        } else {
          merged[key] = sourceValue;
        }
      } else {
        merged[key] = sourceValue;
      }
    }
  }
  return merged;
}
function deepEquals(a, b) {
  if (a === b)
    return true;
  if (Array.isArray(a) && Array.isArray(b)) {
    if (a.length !== b.length)
      return false;
    for (let i = 0;i < a.length; i++) {
      if (!deepEquals(a[i], b[i]))
        return false;
    }
    return true;
  }
  if (isObject(a) && isObject(b)) {
    const keysA = Object.keys(a);
    const keysB = Object.keys(b);
    if (keysA.length !== keysB.length)
      return false;
    for (const key of keysA) {
      if (!Object.prototype.hasOwnProperty.call(b, key))
        return false;
      if (!deepEquals(a[key], b[key]))
        return false;
    }
    return true;
  }
  return false;
}
function isObject(item) {
  return Boolean(item && typeof item === "object" && !Array.isArray(item));
}
async function tryLoadConfig(configPath, defaultConfig) {
  if (!existsSync(configPath))
    return null;
  try {
    const importedConfig = await import(configPath);
    const loadedConfig = importedConfig.default || importedConfig;
    if (typeof loadedConfig !== "object" || loadedConfig === null || Array.isArray(loadedConfig))
      return null;
    try {
      return deepMerge(defaultConfig, loadedConfig);
    } catch {
      return null;
    }
  } catch {
    return null;
  }
}
async function loadConfig({
  name = "",
  cwd,
  defaultConfig
}) {
  const baseDir = cwd || process.cwd();
  const extensions = [".ts", ".js", ".mjs", ".cjs", ".json"];
  const configPaths = [
    `${name}.config`,
    `.${name}.config`,
    name,
    `.${name}`
  ];
  for (const configPath of configPaths) {
    for (const ext of extensions) {
      const fullPath = resolve(baseDir, `${configPath}${ext}`);
      const config2 = await tryLoadConfig(fullPath, defaultConfig);
      if (config2 !== null)
        return config2;
    }
  }
  console.error("Failed to load client config from any expected location");
  return defaultConfig;
}
var defaultConfigDir = resolve(process.cwd(), "config");
var defaultGeneratedDir = resolve(process.cwd(), "src/generated");

// src/utils/activation.ts
import { execSync } from "node:child_process";
import { existsSync as existsSync2, readFileSync, writeFileSync as writeFileSync2 } from "node:fs";
import { homedir } from "node:os";
import path from "node:path";
import process2 from "node:process";
var ACTIVATION_LOCATIONS = {
  win32: [
    path.join(homedir(), "AppData", "Roaming", "Audible", "system.cfg"),
    path.join(homedir(), "AppData", "Local", "Audible", "system.cfg"),
    path.join(homedir(), "AppData", "Roaming", "Audible", "Activation.sys")
  ],
  darwin: [
    path.join(homedir(), "Library", "Application Support", "Audible", "system.cfg"),
    path.join(homedir(), "Library", "Preferences", "com.audible.application")
  ],
  linux: [
    path.join(homedir(), ".config", "audible", "system.cfg"),
    path.join(homedir(), ".audible", "activation.json")
  ]
};
var ACTIVATION_CACHE_FILE = path.join(homedir(), ".aax-activation-cache.json");
function findActivationCode() {
  const cachedCode = loadCachedActivationCode();
  if (cachedCode) {
    return cachedCode;
  }
  const systemCode = findSystemActivationCode();
  if (systemCode) {
    saveActivationCode(systemCode);
    return systemCode;
  }
  return null;
}
function findSystemActivationCode() {
  const platform = process2.platform;
  const locations = ACTIVATION_LOCATIONS[platform] || [];
  for (const location of locations) {
    if (existsSync2(location)) {
      try {
        const content = readFileSync(location, "utf8");
        const patterns = [
          /activation_bytes\s*=\s*([0-9a-fA-F]+)/,
          /ActivationBytes\s*=\s*([0-9a-f]+)/i,
          /"player_key"\s*:\s*"([0-9a-fA-F]+)"/,
          /key="([0-9a-fA-F]+)"/
        ];
        for (const pattern of patterns) {
          const match = content.match(pattern);
          if (match && match[1] && isValidActivationCode(match[1])) {
            return match[1];
          }
        }
        if (location.toLowerCase().endsWith(".sys")) {
          const buffer = readFileSync(location);
          const hexString = buffer.toString("hex");
          const hexMatches = hexString.match(/([0-9a-f]{8})/gi);
          if (hexMatches) {
            for (const potentialCode of hexMatches) {
              if (isValidActivationCode(potentialCode)) {
                return potentialCode;
              }
            }
          }
        }
      } catch {}
    }
  }
  return null;
}
function getActivationBytesFromAudibleCli() {
  try {
    const projectRoot = process2.cwd();
    const audibleBinPath = path.join(projectRoot, "audible");
    if (!existsSync2(audibleBinPath)) {
      console.warn("Audible CLI binary not found in project root");
      return null;
    }
    try {
      execSync(`chmod +x "${audibleBinPath}"`);
    } catch {}
    console.warn("Attempting to get activation bytes from Audible CLI...");
    if (!existsSync2(path.join(homedir(), ".audible", "config.toml"))) {
      console.warn("Audible CLI is not configured. Setting up...");
      try {
        console.warn("Starting Audible CLI quickstart. Please follow the prompts to log in to your Audible account.");
        execSync(`"${audibleBinPath}" quickstart`, { stdio: "inherit", timeout: 120000 });
      } catch (error) {
        console.error("Failed to run Audible CLI quickstart:", error);
        console.warn("You may need to manually run: ./audible quickstart");
        return null;
      }
    }
    try {
      console.warn("Fetching activation bytes from Audible server...");
      const output = execSync(`"${audibleBinPath}" activation-bytes`, { encoding: "utf8", timeout: 30000 }).toString().trim();
      const lines = output.split(`
`);
      const lastLine = lines[lines.length - 1].trim();
      if (isValidActivationCode(lastLine)) {
        console.warn(`✅ Found activation bytes from Audible CLI: ${lastLine.substring(0, 2)}******`);
        return lastLine;
      } else {
        const match = output.match(/([0-9a-f]{8})/i);
        if (match && match[1] && isValidActivationCode(match[1])) {
          console.warn(`✅ Found activation bytes from Audible CLI: ${match[1].substring(0, 2)}******`);
          return match[1];
        }
      }
      console.warn("Could not find valid activation bytes in Audible CLI output.");
      console.warn("Output was:", output);
    } catch (error) {
      console.error("Failed to get activation bytes from Audible CLI:", error);
    }
  } catch (error) {
    console.error("Error using Audible CLI:", error);
  }
  return null;
}
function saveActivationCode(code) {
  try {
    const data = { activationCode: code, timestamp: Date.now() };
    writeFileSync2(ACTIVATION_CACHE_FILE, JSON.stringify(data, null, 2));
  } catch {}
}
function loadCachedActivationCode() {
  try {
    if (existsSync2(ACTIVATION_CACHE_FILE)) {
      const content = readFileSync(ACTIVATION_CACHE_FILE, "utf8");
      const data = JSON.parse(content);
      if (data.activationCode && isValidActivationCode(data.activationCode)) {
        const cacheAge = Date.now() - (data.timestamp || 0);
        const maxAge = 30 * 24 * 60 * 60 * 1000;
        if (cacheAge < maxAge) {
          return data.activationCode;
        }
      }
    }
  } catch {}
  return null;
}
function isValidActivationCode(code) {
  return /^[0-9a-f]{8}$/i.test(code);
}

// src/config.ts
var defaultConfig = {
  verbose: true,
  outputFormat: "mp3",
  outputDir: "./converted",
  chaptersEnabled: true,
  bitrate: 128,
  activationCode: findActivationCode() || undefined
};
var config = await loadConfig({
  name: "aax",
  defaultConfig
});
// src/converter.ts
import { existsSync as existsSync4, mkdirSync as mkdirSync2 } from "node:fs";
import path2 from "node:path";

// src/utils/ffmpeg.ts
import { spawn } from "node:child_process";
import { existsSync as existsSync3 } from "node:fs";
async function checkFFmpeg() {
  const ffmpegPath = config.ffmpegPath || "ffmpeg";
  try {
    if (config.ffmpegPath && !existsSync3(config.ffmpegPath)) {
      return false;
    }
    const process3 = spawn(ffmpegPath, ["-version"]);
    return new Promise((resolve2) => {
      process3.on("close", (code) => {
        resolve2(code === 0);
      });
    });
  } catch {
    return false;
  }
}
async function runFFmpeg(args) {
  const ffmpegPath = config.ffmpegPath || "ffmpeg";
  let outputData = "";
  try {
    const process3 = spawn(ffmpegPath, args);
    process3.stdout.on("data", (data) => {
      outputData += data.toString();
    });
    process3.stderr.on("data", (data) => {
      outputData += data.toString();
      if (config.verbose) {
        console.error(data.toString());
      }
    });
    return new Promise((resolve2) => {
      process3.on("close", (code) => {
        resolve2({
          success: code === 0,
          output: outputData
        });
      });
    });
  } catch (error) {
    return {
      success: false,
      output: error instanceof Error ? error.message : String(error)
    };
  }
}
async function extractAAXMetadata(filePath) {
  const args = [
    "-i",
    filePath,
    "-f",
    "ffmetadata"
  ];
  const { success, output } = await runFFmpeg(args);
  return success ? output : "";
}

// src/utils/metadata.ts
function parseMetadata(metadataText) {
  const metadata = {};
  const lines = metadataText.split(`
`);
  for (const line of lines) {
    if (line.startsWith("title=")) {
      metadata.title = line.substring(6).trim();
    } else if (line.startsWith("artist=")) {
      metadata.author = line.substring(7).trim();
    } else if (line.startsWith("album_artist=")) {
      metadata.narrator = line.substring(13).trim();
    }
  }
  return metadata;
}
function extractChapters(metadataText) {
  const chapters = [];
  const lines = metadataText.split(`
`);
  let inChapter = false;
  let currentChapter = {};
  for (const line of lines) {
    if (line === "[CHAPTER]") {
      inChapter = true;
      currentChapter = {};
    } else if (line === "[/CHAPTER]") {
      inChapter = false;
      if (currentChapter.title && currentChapter.startTime !== undefined && currentChapter.endTime !== undefined) {
        chapters.push(currentChapter);
      }
    } else if (inChapter) {
      if (line.startsWith("TITLE=")) {
        currentChapter.title = line.substring(6).trim();
      } else if (line.startsWith("START=")) {
        currentChapter.startTime = Number.parseFloat(line.substring(6)) / 1000;
      } else if (line.startsWith("END=")) {
        currentChapter.endTime = Number.parseFloat(line.substring(4)) / 1000;
      }
    }
  }
  return chapters;
}
async function getBookMetadata(filePath) {
  const metadataText = await extractAAXMetadata(filePath);
  const basicMetadata = parseMetadata(metadataText);
  const chapters = extractChapters(metadataText);
  return {
    ...basicMetadata,
    chapters
  };
}

// src/converter.ts
async function convertAAX(options) {
  const outputFormat = options.outputFormat || config.outputFormat || "mp3";
  const outputDir = options.outputDir || config.outputDir || ".";
  const activationCode = options.activationCode || config.activationCode;
  const chaptersEnabled = options.chaptersEnabled ?? config.chaptersEnabled ?? false;
  const bitrate = options.bitrate || config.bitrate || 64;
  if (!existsSync4(options.inputFile)) {
    return {
      success: false,
      error: `Input file does not exist: ${options.inputFile}`
    };
  }
  if (!existsSync4(outputDir)) {
    try {
      mkdirSync2(outputDir, { recursive: true });
    } catch (error) {
      return {
        success: false,
        error: `Could not create output directory: ${error.message}`
      };
    }
  }
  const ffmpegAvailable = await checkFFmpeg();
  if (!ffmpegAvailable) {
    return {
      success: false,
      error: "FFmpeg is not available. Please install FFmpeg or set ffmpegPath in config."
    };
  }
  if (!activationCode) {
    return {
      success: false,
      error: "No activation code provided. This is required to convert AAX files."
    };
  }
  console.warn(`Using activation code: ${activationCode.substring(0, 2)}******`);
  try {
    const metadata = await getBookMetadata(options.inputFile);
    const baseName = metadata.title ? `${metadata.title.replace(/[\\/:*?"<>|]/g, "_")}` : path2.basename(options.inputFile, path2.extname(options.inputFile));
    const outputPath = path2.join(outputDir, `${baseName}.${outputFormat}`);
    const ffmpegArgs = [];
    ffmpegArgs.push("-activation_bytes", activationCode);
    ffmpegArgs.push("-i", options.inputFile);
    ffmpegArgs.push("-ab", `${bitrate}k`);
    ffmpegArgs.push("-map_metadata", "0");
    if (chaptersEnabled) {
      ffmpegArgs.push("-map_chapters", "0");
    }
    if (outputFormat === "mp3") {
      ffmpegArgs.push("-codec:a", "libmp3lame");
    } else if (outputFormat === "m4a" || outputFormat === "m4b") {
      ffmpegArgs.push("-codec:a", "aac");
      ffmpegArgs.push("-f", "mp4");
    }
    ffmpegArgs.push(outputPath);
    console.warn(`FFmpeg command: ffmpeg ${ffmpegArgs.join(" ")}`);
    const { success, output } = await runFFmpeg(ffmpegArgs);
    if (success) {
      return {
        success: true,
        outputPath
      };
    } else {
      console.warn("First conversion attempt failed, trying with lowercase activation code...");
      ffmpegArgs[1] = activationCode.toLowerCase();
      console.warn(`FFmpeg command (with lowercase code): ffmpeg ${ffmpegArgs.join(" ")}`);
      const retryResult = await runFFmpeg(ffmpegArgs);
      if (retryResult.success) {
        return {
          success: true,
          outputPath
        };
      }
      return {
        success: false,
        error: `FFmpeg conversion failed: ${output}`
      };
    }
  } catch (error) {
    return {
      success: false,
      error: `Error during conversion: ${error.message}`
    };
  }
}
async function splitToChapters(options) {
  const chaptersEnabled = true;
  return convertAAX({ ...options, chaptersEnabled });
}
export {
  splitToChapters,
  isValidActivationCode,
  getActivationBytesFromAudibleCli,
  convertAAX,
  config
};
